@@ -3,9 +3,10 @@
 """
 import argparse
 from os.path import join as opj
+from os.path import dirname
 from biobb_structure_checking.param_input import Dialog
 
-VERSION = '3.8.1'
+VERSION = '3.9.1'
 
 # Default locations and settings
 DATA_DIR_DEFAULT_PATH = 'dat'
@@ -24,11 +25,22 @@ DEFAULTS = {
     'atom_limit': 1000000,
     'mem_check': False,
     'rename_terms': False,
+    'verbose': False,
+    'debug': False,
     'options' : ''
 }
 
-def set_defaults(base_dir_path, args):
-    """ Checks input args and complete with defaults if necessary """
+def set_defaults(base_dir_path, args=None):
+    """
+    | Constants set_defaults
+    | Checks input args and complete with defaults if necessary
+
+    Args:
+        base_dir_path (str) : Directory where application resides
+        args (dict) : Arguments as passed from the command line
+    """
+    if args is None:
+        args = {}
 
     data_dir_path = opj(base_dir_path, DATA_DIR_DEFAULT_PATH)
 
@@ -53,7 +65,7 @@ def set_defaults(base_dir_path, args):
 
     return args
 
-# Main Command Line
+# Main Command Line Management
 CMD_LINE = argparse.ArgumentParser(
     description='Basic Structure checking based on MDWeb'
 )
@@ -113,14 +125,14 @@ CMD_LINE.add_argument(
 CMD_LINE.add_argument(
     '-o', '--output',
     dest='output_structure_path',
-    help='Output structure. Format PDB'
+    help='Output structure. Format PDB|PDBQT|PQR|CMIP'
 )
 
 CMD_LINE.add_argument(
     '--rename_terms',
     action="store_true",
     dest='rename_terms',
-    help='Renames terminal residues to NXXX, CXXX'
+    help='Show terminal residues as NXXX, CXXX in output files'
 )
 CMD_LINE.add_argument(
     '--json',
@@ -203,53 +215,53 @@ DIALOGS = Dialog()
 #Multiple parameters should come as separate lines with a unique "command"
 
 DIALOGS.add_entry('command_list', 'Runs a list of commands')
-DIALOGS.add_option('command_list', '--list', 'op_list', 'Command List File')
+DIALOGS.add_option('command_list', '--list', 'op_list', 'Command List File or string (; separated)')
 
 DIALOGS.add_entry('models', 'Checks and selects models')
-DIALOGS.add_option('models', '--select', 'select_model', \
+DIALOGS.add_option('models', '--select', 'select', \
     'Select model to keep', int)
 
 DIALOGS.add_entry('chains', 'Checks and selects chains')
-DIALOGS.add_option('chains', '--select', 'select_chains',\
+DIALOGS.add_option('chains', '--select', 'select',\
     'Chains (All | protein | na | dna | rna | Chain list comma separated)')
 
 DIALOGS.add_entry('altloc', 'Checks and selects alternative locations')
-DIALOGS.add_option('altloc', '--select', 'select_altloc', \
+DIALOGS.add_option('altloc', '--select', 'select', \
     'Select altloc occupancy|alt_id')
 
 DIALOGS.add_entry('inscodes', 'Checks residues with insertion codes')
 DIALOGS.add_option('inscodes', '--renum', 'renum', 'Renumber residues', 'bool')
 
 DIALOGS.add_entry('metals', 'Checks and optionally removes metal atoms (will be deprecated in v1.1')
-DIALOGS.add_option('metals', '--remove', 'remove_metals', 'Remove Metal ions')
+DIALOGS.add_option('metals', '--remove', 'remove', 'Remove Metal ions')
 
 DIALOGS.add_entry('water', 'Checks and optionally removes water molecules')
-DIALOGS.add_option('water', '--remove', 'remove_wat', 'Remove All Water molecules')
+DIALOGS.add_option('water', '--remove', 'remove', 'Remove All Water molecules')
 
 DIALOGS.add_entry('ligands', 'Checks and optionally removes ligand residues'\
     ' (will be deprecated)')
-DIALOGS.add_option('ligands', '--remove', 'remove_ligands', 'Remove Ligand residues')
+DIALOGS.add_option('ligands', '--remove', 'remove', 'Remove Ligand residues')
 
 DIALOGS.add_entry('rem_hydrogen', 'Checks and optionally removes hydrogen atoms')
-DIALOGS.add_option('rem_hydrogen', '--remove', 'remove_h', 'Remove Hydrogen atoms')
+DIALOGS.add_option('rem_hydrogen', '--remove', 'remove', 'Remove Hydrogen atoms')
 
 DIALOGS.add_entry('amide', 'Checks and optionally fixes wrong amide contacts')
-DIALOGS.add_option('amide', '--fix', 'amide_fix', 'Fix Residues (All | None | List)')
+DIALOGS.add_option('amide', '--fix', 'fix', 'Fix Residues (All | None | List)')
 DIALOGS.add_option('amide', '--no_recheck', 'no_recheck',\
     'Re-check after modification', 'bool')
 
 DIALOGS.add_entry('chiral', 'Checks and optionally fixes side chains with wrong chirality')
-DIALOGS.add_option('chiral', '--fix', 'chiral_fix', 'Fix Residues (All | None | List)')
+DIALOGS.add_option('chiral', '--fix', 'fix', 'Fix Residues (All | None | List)')
 DIALOGS.add_option('chiral', '--no_check_clashes', 'no_check_clashes',\
     'Do not check for new clashes', 'bool')
 
 DIALOGS.add_entry('chiral_bck', 'Checks residues with wrong CA quiral')
-#DIALOGS.add_option('chiral_bck', '--fix', 'chiral_fix', 'Fix Residues (All | None | List)')
+#DIALOGS.add_option('chiral_bck', '--fix', 'fix', 'Fix Residues (All | None | List)')
 
 DIALOGS.add_entry('fixside', 'Checks and fixes missing side chain atoms')
-DIALOGS.add_option('fixside', '--fix', 'fix_side',\
+DIALOGS.add_option('fixside', '--fix', 'fix',\
     'Add missing atoms to side chains (All | None | List)')
-DIALOGS.add_option('fixside', '--no_rem', 'no_rem_extra',\
+DIALOGS.add_option('fixside', '--no_rem_extra', 'no_rem_extra',\
     'Do not remove unknown atoms', 'bool')
 DIALOGS.add_option('fixside', '--no_check_clashes', 'no_check_clashes',\
     'Do not check for new clashes', 'bool')
@@ -257,9 +269,9 @@ DIALOGS.add_option('fixside', '--rebuild', 'rebuild',\
     'Rebuild complete side chain', 'bool')
 
 DIALOGS.add_entry('backbone', 'Checks and fixes several backbone issues')
-DIALOGS.add_option('backbone', '--fix_atoms', 'fix_back',\
+DIALOGS.add_option('backbone', '--fix_atoms', 'fix_atoms',\
     'Add missing Oxygen atoms to backbone (All | None | List)')
-DIALOGS.add_option('backbone', '--fix_chain', 'fix_main',\
+DIALOGS.add_option('backbone', '--fix_chain', 'fix_chain',\
     'Fixes missing main chain segments (All | None | List)')
 DIALOGS.add_option('backbone', '--add_caps', 'add_caps',\
     'Adds ACE and NME caps to missing main chain segments (All | None)')
@@ -271,7 +283,7 @@ DIALOGS.add_option('backbone', '--no_recheck', 'no_recheck',\
     'Do not re-check after modification', 'bool')
 
 DIALOGS.add_entry('mutateside', 'Performs side chain mutations')
-DIALOGS.add_option('mutateside', '--mut', 'mut_list',\
+DIALOGS.add_option('mutateside', '--mut', 'mut',\
     'Mutate side chains (Mutation List as [*:]arg234Thr)')
 DIALOGS.add_option('mutateside', '--no_check_clashes', 'no_check_clashes',\
     'Do not check for generated clashes', 'bool')
@@ -280,24 +292,23 @@ DIALOGS.add_option('mutateside', '--rebuild', 'rebuild',\
 DIALOGS.add_option('mutateside', '--na_seq', 'na_seq',\
     'Mutate DNA duplex to generate sequence')
 DIALOGS.add_entry('add_hydrogen', 'Add hydrogen atoms with tautomer/ion selection')
-DIALOGS.add_option('add_hydrogen', '--add_mode', 'mode',\
+DIALOGS.add_option('add_hydrogen', '--add_mode', 'add_mode',\
     'Selection mode (None | auto | list | ph | int | int_his )')
-DIALOGS.add_option('add_hydrogen', '--pH', 'pH_value',\
+DIALOGS.add_option('add_hydrogen', '--pH', 'pH',\
     'pH (0-14)')
-DIALOGS.add_option('add_hydrogen', '--list', 'ions_list',\
+DIALOGS.add_option('add_hydrogen', '--list', 'list',\
     'Ionic form selection (list as [*:]his234hip)')
 DIALOGS.add_option('add_hydrogen', '--no_fix_side', 'no_fix_side',\
     'Do not fix side chains', 'bool')
 DIALOGS.add_option('add_hydrogen', '--keep_h', 'keep_h',\
     'Keep original hydrogen atoms', 'bool')
 DIALOGS.add_option('add_hydrogen', '--add_charges', 'add_charges',\
-    'Update atom partial charges', 'bool')
-
+    'Update atom partial charges and add atom types from given forcefield (ADT|CMIP)', default="")
 DIALOGS.add_entry('clashes', 'Checks atom clashes')
 #DIALOGS.add_option('clashes', '--no_wat', 'discard_wat', 'Discard water molecules', 'bool')
 
 DIALOGS.add_entry('getss', 'Checks SS bonds by distance')
-DIALOGS.add_option('getss', '--mark', 'getss_mark', 'Mark Cys pairs as SS bond (All | None | List)')
+DIALOGS.add_option('getss', '--mark', 'mark', 'Mark Cys pairs as SS bond (All | None | List)')
 
 DIALOGS.add_entry('cistransbck', 'Checks or cis peptide bonds')
 DIALOGS.add_entry('checkall', 'Runs all checks, no modification')
@@ -305,13 +316,12 @@ DIALOGS.add_entry('fixall', 'Fix all found issues with default options')
 
 DIALOGS.add_entry('sequences', 'Print Canonical and Structure sequences on FASTA format')
 
-
+# All methods to perform checkall
 AVAILABLE_METHODS = [
     'models', 'chains', 'inscodes', 'altloc', 'rem_hydrogen', 'add_hydrogen',
     'water', 'metals', 'ligands', 'getss', 'amide', 'chiral', 'chiral_bck',
     'fixside', 'backbone', 'cistransbck', 'clashes', 'sequences']
 
-
 MSGS = {
     #management
     'NON_MODIFIED_STRUCTURE': 'Structure not modified, not saving. Override with --force_save',
@@ -420,5 +430,26 @@ MSGS = {
     'STRUCTURE_LOADED': 'Structure {} loaded',
     #NA related
     'NO_NA': 'No NA chains found, skipping',
-    'WARN_NOBUILD_NA': 'Warning: --rebuild only available for protein chains'
+    'WARN_NOBUILD_NA': 'Warning: --rebuild only available for protein chains',
+    # Model utils
+    'ATOM_NOT_FOUND': 'Warning: atom {:3} not found in {}',
+    'NO_BACKBONE_ATOMS': 'Warning: No backbone atoms defined',
+    'RESIDUE_NOT_VALID' : "Warning: Residue not valid in this context ",
+    'NOT_ENOUGH_ATOMS' : "Warning: not enough atoms to build {} hydrogen atoms on"
 }
+
+# Help handler
+def help(command=None):
+    """
+    | constants help
+    | Handler for getting help on commands
+
+    Args:
+        command (str) : (None) Command requested, if empty help on all commands is provided.
+    """
+    if not command:
+        help_path = opj(dirname(__file__), DATA_DIR_DEFAULT_PATH, COMMANDS_HELP_PATH)
+        with open(help_path) as help_file:
+            print(help_file.read())
+    else:
+        DIALOGS.get_parameter(command, '', print_help=True)
