@@ -119,8 +119,12 @@ class StructureManager:
         self.biounit = False
         self.fixed_side = False
         self.file_format = file_format
+        self.has_charges = False
 
         self.data_library = DataLibManager(data_library_path)
+        for ff in self.data_library.ff_data:
+            self.data_library.get_ff_data(os.path.dirname(data_library_path) + '/' + ff  + '_prm.json')
+
         self.res_library = ResidueLib(res_library_path)
 
 
@@ -164,9 +168,12 @@ class StructureManager:
         else:
             real_pdb_path = input_pdb_path
 
-        if '.pdb' in real_pdb_path:
-            parser = PDBParser(PERMISSIVE=1)
+        if '.pdb' in real_pdb_path: 
+            parser = PDBParser(PERMISSIVE=1, is_pqr=False)
             input_format = 'pdb'
+        elif '.pqr' in real_pdb_path: 
+            parser = PDBParser(PERMISSIVE=1, is_pqr=True)
+            input_format = 'pqr'
         elif '.cif' in real_pdb_path:
             parser = MMCIFParser()
             input_format = 'cif'
@@ -181,10 +188,11 @@ class StructureManager:
             raise ParseError('ValueError', err)
         except PDBConstructionException as err:
             raise ParseError('PDBBuildError', err)
-        if input_format == 'pdb':
+        if input_format in ['pdb', 'pqr']:
             self.headers = parse_pdb_header(real_pdb_path)
         else:
             self.headers = MMCIF2Dict(real_pdb_path)
+            
         return input_format
 
     def update_internals(self, cif_warn: bool = False):
@@ -227,50 +235,67 @@ class StructureManager:
         """ Sets  Bio.PDB.Atom.serial_number for all atoms in the structure,
             overriding original if any.
         """
+        if self.input_format == 'pqr' or self.has_charges:
+            self.total_charge = 0.
+        else:
+            self.total_charge = None
         i = 1
         for atm in self.st.get_atoms():
             atm.serial_number = i
             if hasattr(atm, 'selected_child'):
                 atm.selected_child.serial_number = i
+            if atm.pqr_charge is not None and self.total_charge is not None:
+                self.total_charge += atm.pqr_charge
             i += 1
+        #self.has_charges = (self.total_charge is not None)
 
-    def update_atom_charges(self):
-        """ Updats atom charges from data library """
+    def update_atom_charges(self, ff):
+        """ Update atom charges and types from data library """
+        
         print("Updating partial charges and atom types")
-        tot_chrg = 0
+        
+        self.total_charge = 0.
+        
+        if ff not in self.data_library.ff_data:
+            raise UnknownFFError(ff)
+        ff_data = self.data_library.ff_data[ff]
+
         for res in self.st.get_residues():
-            rcode = self.data_library.get_canonical_resname(res.get_resname())
+            rcode = res.get_resname()
             if len(rcode) == 4:
                 rcode3 = rcode[1:]
             else:
                 rcode3 = rcode
-            rcode3 = self.data_library.get_canonical_resname(rcode3)
+            can_rcode3 = self.data_library.get_canonical_resname(rcode3)
             if rcode not in self.res_library.residues:
                 print("Warning: {} not found in residue library atom charges set to 0.".format(rcode))
                 for atm in res.get_atoms():
-                    atm.charge = 0.
+                    atm.pqr_charge = 0.
+                    atm.radius = 0.
                     if atm.id in self.data_library.atom_data['metal_atoms']:
-                        atm.ADT_type = atm.id.lower().capitalize()
+                        atm.xtra['atom_type'] = atm.id.lower().capitalize()
                     else:
-                        atm.ADT_type = atm.element
+                        atm.xtra['atom_type'] = atm.element
             else:
                 oxt_ok = rcode[0] != 'C' or len(rcode) != 4
                 res_chr = 0.
+                
                 for atm in res.get_atoms():
-                    atm.charge = self.res_library.get_atom_def(rcode, atm.id).chrg
-                    if atm.id in self.data_library.residue_data[rcode3]['ADT_type']:
-                        atm.ADT_type = self.data_library.residue_data[rcode3]['ADT_type'][atm.id]
-                    elif atm.id in self.data_library.residue_data['*']['ADT_type']:
-                        atm.ADT_type = self.data_library.residue_data['*']['ADT_type'][atm.id]
+                    atm.pqr_charge = self.res_library.get_atom_def(rcode, atm.id).chrg
+                    if atm.id in ff_data['residue_data'][can_rcode3]:
+                        atm.xtra['atom_type'] = ff_data['residue_data'][can_rcode3][atm.id]
+                    elif atm.id in ff_data['residue_data']['*']:
+                        atm.xtra['atom_type'] = ff_data['residue_data']['*'][atm.id]
                     else:
-                        atm.ADT_type = atm.element
-                    res_chr += atm.charge
-                    tot_chrg += atm.charge
+                        atm.xtra['atom_type'] = atm.element
+                    atm.radius = ff_data['rvdw'][atm.xtra['atom_type']]
+                    res_chr += atm.pqr_charge
+                    self.total_charge += atm.pqr_charge
                     if atm.id == 'OXT':
                         oxt_ok = True
                 if not oxt_ok:
-                    print("Warning: OXT atom missing in {}. Run backbone --add_atoms first".format(mu.residue_id(res)))
-        print("Total assigned charge: {:10.2f}".format(tot_chrg))
+                    print("Warning: OXT atom missing in {}. Run backbone --fix_atoms first".format(mu.residue_id(res)))
+        print("Total assigned charge: {:10.2f}".format(self.total_charge))
 
         self.has_charges = True
 
@@ -398,17 +423,6 @@ class StructureManager:
             severe='severe' in contact_types
         )
 
-    def check_rr_clashes(self, res1: Residue, res2: Residue, contact_types: Iterable[str]) -> Dict[str, Tuple[Residue, Residue, float]]:
-        """ Checks all clashes between two residues"""
-        return mu.check_rr_clashes(
-            res1,
-            res2,
-            self.data_library.distances['CLASH_DIST'],
-            self.data_library.get_atom_lists(contact_types),
-            severe='severe' in contact_types
-
-        )
-
     def check_missing_atoms(self) -> List[Tuple[Residue, Dict[str, List[str]]]]:
         """ Makes a **list of missing atoms** in the structure
 
@@ -482,12 +496,15 @@ class StructureManager:
         """
         ion_res = self.data_library.ion_res
         hydrogen_lists = self.data_library.get_hydrogen_atoms()
-
+       
         ion_res_list = []
         for res in self.all_residues:
             rcode = res.get_resname()
+            if len(rcode) == 4:
+                rcode = rcode[1:]
             if rcode in ion_res:
                 ion_res_list.append((res, hydrogen_lists[rcode]))
+
         return ion_res_list
 
     def get_backbone_breaks(self) -> Dict[str, List[List[Residue]]]:
@@ -636,7 +653,8 @@ class StructureManager:
             'res_ligands': self.res_ligands,
             'num_wat': self.num_wat,
             'ca_only': self.ca_only,
-            'biounit': self.biounit
+            'biounit': self.biounit,
+            'total_charge': self.total_charge
         }
 
     def get_term_res(self) -> List[Tuple[str, Residue]]:
@@ -754,8 +772,10 @@ class StructureManager:
             print('Small mol ligands found')
             for res in self.hetatm[mu.ORGANIC]:
                 print(mu.residue_id(res))
+        if self.has_charges:
+            print("Total charge {:6.3f}".format(self.total_charge))
 
-    def save_structure(self, output_pdb_path: str, mod_id: str = None, rename_terms: bool = False):
+    def save_structure(self, output_pdb_path: str, mod_id: str = None, rename_terms: bool = False, output_format='pdb'):
         """
         Saves structure on disk in PDB format
 
@@ -768,10 +788,12 @@ class StructureManager:
         """
         if not output_pdb_path:
             raise OutputPathNotProvidedError
-        pdbio = PDBIO_extended()
+        pdbio = PDBIO_extended(is_pqr=self.has_charges, output_format=output_format)
 
         if rename_terms:
             self.rename_terms(self.get_term_res())
+        else:
+            self.revert_terms()
 
         if mod_id is None:
             pdbio.set_structure(self.st)
@@ -1210,18 +1232,20 @@ class StructureManager:
                 'OXT',
                 mu.build_coords_SP2(mu.OINTERNALS[0], res['C'], res['CA'], res['O'])
             )
+            res.resname = 'C' + res.resname
 
         self.atom_renumbering()
         self.modified = True
         return True
 
-    def add_hydrogens(self, ion_res_list, remove_h: bool = True, add_charges: bool = False):
+    def add_hydrogens(self, ion_res_list, remove_h: bool = True, add_charges: str = 'ADT'):
         """
         Add hydrogens considering selections in ion_res_list
 
         Args:
            **r_at_list**: dict as Bio.PDB.Residue: Tauromeric Option
            **remove_h**: Remove Hydrogen atom before adding new ones
+           **add_charges**: Add charges and atom types acording to ff
         """
         add_h_rules = self.data_library.get_add_h_rules()
 
@@ -1249,18 +1273,23 @@ class StructureManager:
 
             rcode = res.get_resname()
 
+            if len(rcode) == 4:
+                rcode = rcode[1:]
+
             if rcode == 'GLY':
                 continue
             # Fixed for modified residues already in the original PDB
+        
             if rcode in self.data_library.canonical_codes:
                 rcode_can = self.data_library.canonical_codes[rcode]
             else:
                 rcode_can = rcode
 
-            if rcode_can not in add_h_rules:
+            
+            if rcode_can not in add_h_rules:              
                 print(NotAValidResidueError(rcode).message)
                 continue
-
+            
             if rcode_can == rcode:
                 h_rules = add_h_rules[rcode]
             else:
@@ -1287,10 +1316,10 @@ class StructureManager:
                 print(error_msg, mu.residue_id(res))
 
         self.residue_renumbering()
-        self.atom_renumbering()
         if add_charges:
             self.rename_terms(self.get_term_res())
-            self.update_atom_charges()
+            self.update_atom_charges(add_charges)
+        self.atom_renumbering()
         self.modified = True
 
     def mark_ssbonds(self, cys_list):
@@ -1307,6 +1336,12 @@ class StructureManager:
             if t[1].resname not in ('ACE', 'NME'):
                 t[1].resname = t[0] + t[1].resname
 
+    def revert_terms(self):
+        """ Reverts 4 char len residue names to 3 letter codes"""
+        for res in self.st.get_residues():
+            if len(res.get_resname()) == 4:
+                res.resname = res.resname[1:]
+
     def is_N_term(self, res: Residue) -> bool:
         """ Detects whether it is N terminal residue."""
         return res not in self.prev_residue
@@ -1464,3 +1499,7 @@ class NotEnoughAtomsError(Error):
 class ParseError(Error):
     def __init__(self, err_id, err_txt):
         self.message = '{} ({}) found when parsing input structure'.format(err_id, err_txt)
+
+class UnknownFFError(Error):
+    def __init__(self, ff):
+        self.message = '{} is not a valid ff for assigning atom types'.format(ff)
\ No newline at end of file
